%bcond_with fulltest

%global sover .2.1.0
%global soshort .2

Summary:	BLAS-like Library Instantiation Software Framework
Name:		blis
Version:	0.7.0
Release:        3%{?dist}
License:	BSD-3-Clause
URL:		https://github.com/flame/blis
Source0:	https://github.com/flame/blis/archive/%version/%name-%version.tar.gz

BuildRequires: make
BuildRequires: perl gcc-gfortran chrpath

BuildRequires:	perl gcc make
BuildRequires:	/usr/bin/python3 gcc-gfortran chrpath

%ifarch x86_64
BuildRequires: memkind-devel
%endif

%description
BLIS is a portable software framework for instantiating
high-performance BLAS-like dense linear algebra libraries.  The
framework was designed to isolate essential kernels of computation
that, when optimized, immediately enable optimized implementations of
most of its commonly used and computationally intensive operations.
While BLIS exports a new BLAS-like API, it also includes a BLAS
compatibility layer which gives application developers access to BLIS
implementations via traditional BLAS routine calls.

This packaging contains automatically-dispatched
architecture-optimized kernels for some targets, notably recent x86_64.

This is the serial version.

%package	devel
Summary:	Development files for %name
Requires:	%name = %version-%release
Requires:	%name-openmp = %version-%release
Requires:	%name-threads = %version-%release
Requires:	%name-serial64 = %version-%release
Requires:	%name-openmp64 = %version-%release
Requires:	%name-threads64 = %version-%release

%description	devel
The %name-devel package contains libraries and header files for
developing applications that use %name.

%package serial64
Summary:	BLAS-like Library Instantiation Software Framework - 64-bit

%description serial64
This is the serial version with a 64-bit integer interface.

%package openmp
Summary:	BLAS-like Library Instantiation Software Framework - OpenMP

%description openmp
This is the OpenMP-parallelized version.

%package openmp64
Summary:	BLAS-like Library Instantiation Software Framework - OpenMP, 64-bit

%description openmp64
This is the OpenMP-parallelized version with a 64-bit integer interface.

%package threads
Summary:	BLAS-like Library Instantiation Software Framework - pthreads

%description threads
This is the pthreads-parallelized version.

%package threads64
Summary:	BLAS-like Library Instantiation Software Framework - pthreads, 64-bit

%description threads64
This is the pthreads-parallelized version with a 64-bit integer interface.

%package srpm-macros
Summary:	BLIS architecture macros
BuildArch:	noarch

%description srpm-macros
BLIS architecture macros.


%prep
%autosetup -p1
echo %sover | awk -F. '{printf("%s\n%s.%s\n", $2,$3,$4)}' >so_version


%build
case %_arch in
x86_64) arch=x86_64 ;;
aarch64) arch=cortexa57 ;;
*) arch=generic ;;
esac

%global confflags --enable-debug=opt --disable-static --enable-shared --enable-verbose-make --enable-cblas
export CFLAGS="$RPM_OPT_FLAGS -O3 -funsafe-math-optimizations" LDFLAGS="%{?__global_ldflags}"
export PYTHON=python3

./configure --prefix=$(pwd)/o %confflags -t openmp $arch
%make_build SOFLAGS="-shared -Wl,-soname=libbliso.so%sover"
make install

./configure --prefix=$(pwd)/p %confflags -t pthreads $arch
%make_build SOFLAGS="-shared -Wl,-soname=libblisp.so%sover"
make install

mkdir -p blisblas{,o,p,64,o64,p64}
for d in o p; do
  cd $d/lib
  f=libblis.so%sover
  mv $f ${f/./$d.}
  ln -s libblis$d.so%sover libblis$d.so
  ln -s libblis$d.so%sover  libblis$d.so%soshort
  rm libblis.*
  cd ../..
  cc -shared -Wl,-soname=libblas.so.3 -L$(pwd)/$d/lib -lblis$d -o blisblas$d/libblas.so.3 $LDFLAGS
  ln -s libblas.so.3 blisblas$d/libblas.so
done

./configure --prefix=$(pwd)/64 %confflags -b 64 $arch
%make_build SOFLAGS="-shared -Wl,-soname=libblis64.so%sover"
make install

./configure --prefix=$(pwd)/o64 %confflags -b 64 -t openmp $arch
%make_build SOFLAGS="-shared -Wl,-soname=libbliso64.so%sover"
make install

./configure --prefix=$(pwd)/p64 %confflags -b 64 -t pthreads $arch
%make_build SOFLAGS="-shared -Wl,-soname=libblisp64.so%sover"
make install

for d in 64 o64 p64; do
  cd $d/lib
  f=libblis.so%sover
  mv $f ${f/./$d.}
  ln -s libblis$d.so%sover libblis$d.so
  ln -s libblis$d.so%sover libblis$d.so%soshort
  rm -f libblis.*
  cd ../..
  cc -shared -Wl,-soname=libblas64.so.3 -L$(pwd)/$d/lib -lblis$d -o blisblas$d/libblas64.so.3 $LDFLAGS
  ln -s libblas64.so.3 blisblas$d/libblas64.so
done

./configure --prefix=$(pwd)/serial %confflags $arch
%make_build
make install
cc -shared -Wl,-soname=libblas.so.3 -L$(pwd)/serial/lib -lblis -o blisblas/libblas.so.3 $LDFLAGS
ln -s libblas.so.3 blisblas/libblas.so


%install
mkdir -p %buildroot%_libdir %buildroot%_includedir

cp -a {serial,o,p}/lib/* %buildroot%_libdir
mv serial/include/blis %buildroot%_includedir
for d in o p; do
  cp -a $d/include/blis %buildroot%_includedir/blis$d
done
cp -a {64,o64,p64}/lib/* %buildroot%_libdir
for d in 64 o64 p64; do
  cp -a $d/include/blis %buildroot%_includedir/blis$d
done
chmod +x %buildroot%_libdir/*.so.*
cp -a blisblas* %buildroot%_libdir
gzip CHANGELOG
chrpath -d %buildroot%_libdir/*.so.*

%global macrosdir %(d=%{_rpmconfigdir}/macros.d; [ -d $d ] || d=%{_sysconfdir}/rpm; echo $d)
mkdir -p %buildroot%macrosdir
cat <<EOF >%buildroot%macrosdir/macros.blis-srpm
%blis_opt_arches x86_64 aarch64
EOF


%check
gfortran -o dblat blastest/src/fortran/dblat3.f -Lblisblas -Lserial/lib -lblas -lblis
LD_LIBRARY_PATH=$(pwd)/serial/lib:$(pwd)/blisblas ./dblat <<+ || { cat dblat3.summ && false; }
'dblat3.summ'
6
'dblat3.snap'
-1
F
T
T
16.0
7
0 1 2 3 7 31 63
3
0.0 1.0 0.7
3
0.0 1.0 1.3
DGEMM  T
DSYMM  F
+

export LD_LIBRARY_PATH=`pwd`/serial/lib
%if %{with fulltest}
%make_build test
%else
%make_build check
%endif



%global docs CHANGELOG.gz CREDITS README.md

%files
%license LICENSE
%doc %docs
%{_libdir}/libblis.so%{soshort}*
%{_libdir}/blisblas

%files openmp
%license LICENSE
%doc %docs
%{_libdir}/libbliso.so%{soshort}*
%{_libdir}/blisblaso

%files threads
%license LICENSE
%doc %docs
%{_libdir}/libblisp.so%{soshort}*
%{_libdir}/blisblasp

%files serial64
%license LICENSE
%doc %docs
%{_libdir}/libblis64.so%{soshort}*
%{_libdir}/blisblas64

%files openmp64
%license LICENSE
%doc %docs
%{_libdir}/libbliso64.so%{soshort}*
%{_libdir}/blisblaso64

%files threads64
%license LICENSE
%doc %docs
%{_libdir}/libblisp64.so%{soshort}*
%{_libdir}/blisblasp64

%files devel
%doc examples
%{_includedir}/*
%{_libdir}/libblis*.so

%files srpm-macros
%{macrosdir}/macros.blis-srpm

%changelog
* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 0.7.0-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 0.7.0-2
- Rebuilt for OpenCloudOS Stream 23

* Thu Mar 16 2023 cunshunxia <cunshunxia@tencent.com> - 0.7.0-1
- initial build
